@@ -24,6 +24,7 @@
 #include <sys/types.h>
 #include <errno.h>
 
+#include <glib.h>
 #include <glib/gi18n.h>
 
 #include <gdk/gdk.h>
@@ -37,8 +38,6 @@
 #endif
 #endif /* HAVE_HILDON */
 
-#include <libgames-support/games-card-theme.h>
-#include <libgames-support/games-card-themes.h>
 #include <libgames-support/games-clock.h>
 #include <libgames-support/games-debug.h>
 #include <libgames-support/games-glib-compat.h>
@@ -60,6 +59,8 @@
 #include "board-noclutter.h"
 #endif
 
+#include "ar-card-theme.h"
+#include "ar-card-themes.h"
 #include "conf.h"
 #include "game.h"
 #include "stats-dialog.h"
@@ -95,13 +96,6 @@
 #define C_(context, string) (_(string))
 #endif
 
-/* On maemo5, the toolbar doesn't have enough space to show a
- * significant amount of text, so we always use the banner here.
- */
-#if defined(HAVE_HILDON) && (defined(HAVE_MAEMO_3) || defined(HAVE_MAEMO_4))
-#define MAEMO_TOOLBAR_BANNER
-#endif
-
 /* On maemo5, there's no hardware key to exit the fullscreen mode. So we show
  * an overlay button to restore normal mode, if the toolbar is hidden too.
  */
@@ -119,11 +113,9 @@ enum
   ACTION_OPTIONS_MENU,
   ACTION_DEAL,
   ACTION_HINT,
+  ACTION_LEAVE_FULLSCREEN,
 #ifdef HAVE_HILDON
   ACTION_ACCEL_UNDO_MOVE,
-#endif
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
-  ACTION_LEAVE_FULLSCREEN,
 #endif
   LAST_ACTION
 };
@@ -140,14 +132,11 @@ struct _AisleriotWindowPrivate
   AisleriotBoard *board;
 #endif
 
-  GamesCardThemes *theme_manager;
-  GamesCardTheme *theme;
+  ArCardThemes *theme_manager;
+  ArCardTheme *theme;
 
 #ifdef HAVE_HILDON
   guint game_message_hash;
-#ifdef MAEMO_TOOLBAR_BANNER
-  GtkLabel *game_message_label;
-#endif /* MAEMO_TOOLBAR_BANNER */
 #else
   GtkStatusbar *statusbar;
   guint game_message_id;
@@ -579,7 +568,7 @@ install_themes_cb (GtkAction *action,
 {
   AisleriotWindowPrivate *priv = window->priv;
 
-  games_card_themes_install_themes (priv->theme_manager,
+  ar_card_themes_install_themes (priv->theme_manager,
                                     GTK_WINDOW (window),
                                     gtk_get_current_event_time ());
 }
@@ -949,13 +938,11 @@ set_fullscreen_actions (AisleriotWindow *window,
   g_object_set (priv->main_menu, "visible", !is_fullscreen, NULL);
 #endif /* HAVE_MAEMO */
 
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
   gtk_action_set_visible (priv->action[ACTION_LEAVE_FULLSCREEN], is_fullscreen);
   g_object_set (gtk_ui_manager_get_widget (priv->ui_manager, "/Toolbar/LeaveFullscreenSep"),
                 "visible", is_fullscreen,
                 "draw", FALSE,
                 NULL);
-#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
 
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (priv->action[ACTION_FULLSCREEN]),
                                 is_fullscreen);
@@ -977,8 +964,6 @@ fullscreen_toggled_cb (GtkToggleAction *action,
   }
 }
 
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
-
 static void
 leave_fullscreen_cb (GtkAction *action,
                      GtkWindow *window)
@@ -986,8 +971,6 @@ leave_fullscreen_cb (GtkAction *action,
   gtk_window_unfullscreen (window);
 }
 
-#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
-
 static void
 clickmove_toggle_cb (GtkToggleAction *action,
                      AisleriotWindow *window)
@@ -1391,7 +1374,7 @@ install_recently_played_menu (AisleriotWindow *window)
 
 static void
 aisleriot_window_take_card_theme (AisleriotWindow *window,
-                                  GamesCardTheme *theme /* adopting */)
+                                  ArCardTheme *theme /* adopting */)
 {
   AisleriotWindowPrivate *priv = window->priv;
 #if GTK_CHECK_VERSION (2, 10, 0)
@@ -1411,7 +1394,7 @@ aisleriot_window_take_card_theme (AisleriotWindow *window,
     const cairo_font_options_t *font_options;
 
     font_options = gdk_screen_get_font_options (gtk_widget_get_screen (widget));
-    games_card_theme_set_font_options (theme, font_options);
+    ar_card_theme_set_font_options (theme, font_options);
   }
 #endif /* GTK+ 2.10.0 */
 
@@ -1423,8 +1406,8 @@ card_theme_changed_cb (GtkToggleAction *action,
                        AisleriotWindow *window)
 {
   AisleriotWindowPrivate *priv = window->priv;
-  GamesCardThemeInfo *current_theme_info = NULL, *new_theme_info;
-  GamesCardTheme *theme;
+  ArCardThemeInfo *current_theme_info = NULL, *new_theme_info;
+  ArCardTheme *theme;
   const char *theme_name;
 
   if (!gtk_toggle_action_get_active (action))
@@ -1434,13 +1417,13 @@ card_theme_changed_cb (GtkToggleAction *action,
   g_assert (new_theme_info != NULL);
 
   if (priv->theme) {
-    current_theme_info = games_card_theme_get_theme_info (priv->theme);
+    current_theme_info = ar_card_theme_get_theme_info (priv->theme);
   }
 
-  if (games_card_theme_info_equal (new_theme_info, current_theme_info))
+  if (ar_card_theme_info_equal (new_theme_info, current_theme_info))
     return;
 
-  theme = games_card_themes_get_theme (priv->theme_manager, new_theme_info);
+  theme = ar_card_themes_get_theme (priv->theme_manager, new_theme_info);
   if (!theme) {
     GSList *group, *l;
 
@@ -1455,13 +1438,13 @@ card_theme_changed_cb (GtkToggleAction *action,
     group = gtk_radio_action_get_group (GTK_RADIO_ACTION (action));
     for (l = group; l != NULL; l = l->next) {
       GtkToggleAction *theme_action = GTK_TOGGLE_ACTION (l->data);
-      GamesCardThemeInfo *info;
+      ArCardThemeInfo *info;
 
       if (theme_action == action)
         continue;
 
       info = g_object_get_data (G_OBJECT (theme_action), "theme-info");
-      if (!games_card_theme_info_equal (info, current_theme_info))
+      if (!ar_card_theme_info_equal (info, current_theme_info))
         continue;
 
       /* The check at the top will prevent an infinite loop */
@@ -1474,18 +1457,18 @@ card_theme_changed_cb (GtkToggleAction *action,
 
   aisleriot_window_take_card_theme (window, theme);
 
-  theme_name = games_card_theme_info_get_persistent_name (new_theme_info);
+  theme_name = ar_card_theme_info_get_persistent_name (new_theme_info);
   games_conf_set_string (NULL, aisleriot_conf_get_key (CONF_THEME), theme_name);
 }
 
 static void
-install_card_theme_menu (GamesCardThemes *theme_manager,
+install_card_theme_menu (ArCardThemes *theme_manager,
                          AisleriotWindow *window)
 {
   AisleriotWindowPrivate *priv = window->priv;
   GList *list, *l;
   GSList *radio_group = NULL;
-  GamesCardThemeInfo *current_theme_info;
+  ArCardThemeInfo *current_theme_info;
   guint i = 0;
 
   /* Clean out the old menu */
@@ -1501,11 +1484,11 @@ install_card_theme_menu (GamesCardThemes *theme_manager,
   /* See gtk bug #424448 */
   gtk_ui_manager_ensure_update (priv->ui_manager);
 
-  list = games_card_themes_get_themes (priv->theme_manager);
+  list = ar_card_themes_get_themes (priv->theme_manager);
 
   /* No need to install the menu when there's only one theme available anyway */
   if (list == NULL || list->next == NULL) {
-    g_list_foreach (list, (GFunc) games_card_theme_info_unref, NULL);
+    g_list_foreach (list, (GFunc) ar_card_theme_info_unref, NULL);
     g_list_free (list);
     return;
   }
@@ -1517,18 +1500,18 @@ install_card_theme_menu (GamesCardThemes *theme_manager,
   priv->card_themes_merge_id = gtk_ui_manager_new_merge_id (priv->ui_manager);
 
   if (priv->theme) {
-    current_theme_info = games_card_theme_get_theme_info (priv->theme);
+    current_theme_info = ar_card_theme_get_theme_info (priv->theme);
   } else {
     current_theme_info = NULL;
   }
 
   for (l = list; l != NULL; l = l->next) {
-    GamesCardThemeInfo *info = (GamesCardThemeInfo *) l->data;
+    ArCardThemeInfo *info = (ArCardThemeInfo *) l->data;
     GtkRadioAction *action;
     char actionname[32];
     char *display_name, *tooltip;
 
-    display_name = g_strdup (games_card_theme_info_get_display_name (info));
+    display_name = g_strdup (ar_card_theme_info_get_display_name (info));
 
     g_snprintf (actionname, sizeof (actionname), "Theme%d", i);
 #ifdef HAVE_HILDON
@@ -1544,13 +1527,13 @@ install_card_theme_menu (GamesCardThemes *theme_manager,
     radio_group = gtk_radio_action_get_group (action);
 
     /* Check if this is the current theme's action. Do this before connecting the callback */
-    if (games_card_theme_info_equal (info, current_theme_info)) {
+    if (ar_card_theme_info_equal (info, current_theme_info)) {
       gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
     }
 
     /* We steal the data from the list */
     g_object_set_data_full (G_OBJECT (action), "theme-info",
-                            l->data, (GDestroyNotify) games_card_theme_info_unref);
+                            l->data, (GDestroyNotify) ar_card_theme_info_unref);
     l->data = NULL;
 
     g_signal_connect (action, "toggled",
@@ -1586,7 +1569,7 @@ view_menu_activate_cb (GtkAction *action,
   /* Request the list of themes. If it wasn't updated yet, the "changed"
    * callback will build the card themes submenu.
    */
-  games_card_themes_request_themes (priv->theme_manager);
+  ar_card_themes_request_themes (priv->theme_manager);
 }
 
 /* Callbacks */
@@ -1756,9 +1739,6 @@ game_type_changed_cb (AisleriotGame *game,
   gtk_action_set_visible (priv->action[ACTION_DEAL], dealable);
 
 #ifdef HAVE_HILDON
-#ifdef MAEMO_TOOLBAR_BANNER
-  gtk_label_set_text (priv->game_message_label, NULL);
-#endif /* MAEMO_TOOLBAR_BANNER */
 #else
   games_clock_reset (GAMES_CLOCK (priv->clock));
 
@@ -1776,22 +1756,18 @@ static void
 game_new_cb (AisleriotGame *game,
              AisleriotWindow *window)
 {
-#if defined(MAEMO_TOOLBAR_BANNER) || !defined(HAVE_HILDON)
+#ifndef HAVE_HILDON
   AisleriotWindowPrivate *priv = window->priv;
 #endif
 
   update_statistics_display (window);
 
-#ifdef HAVE_HILDON
-#ifdef MAEMO_TOOLBAR_BANNER
-  gtk_label_set_text (priv->game_message_label, NULL);
-#endif /* MAEMO_TOOLBAR_BANNER */
-#else
+#ifndef HAVE_HILDON
   games_clock_reset (GAMES_CLOCK (priv->clock));
 
   gtk_statusbar_pop (priv->statusbar, priv->game_message_id);
   gtk_statusbar_pop (priv->statusbar, priv->board_message_id);
-#endif /* HAVE_HILDON */
+#endif /* !HAVE_HILDON */
 }
 
 static void
@@ -1801,16 +1777,8 @@ game_statusbar_message_cb (AisleriotGame *game,
 {
   AisleriotWindowPrivate *priv = window->priv;
 #ifdef HAVE_HILDON
-#ifdef MAEMO_TOOLBAR_BANNER
-  gtk_label_set_text (priv->game_message_label, message);
-#endif
 
-  /* The banners are annoying, but until I get a better idea, use them
-   * when the toolbar is hidden.
-   */
-#ifdef MAEMO_TOOLBAR_BANNER
-  if (!priv->toolbar_visible)
-#endif
+  /* The banners are annoying, but until I get a better idea, use them */
   {
     guint hash;
 
@@ -2118,7 +2086,7 @@ aisleriot_window_style_set (GtkWidget *widget,
     return;
 
   font_options = gdk_screen_get_font_options (gtk_widget_get_screen (widget));
-  games_card_theme_set_font_options (priv->theme, font_options);
+  ar_card_theme_set_font_options (priv->theme, font_options);
 
   /* FIXMEchpe: clear the cached cards in the slots?? */
 }
@@ -2253,10 +2221,8 @@ aisleriot_window_init (AisleriotWindow *window)
 #endif /* ENABLE_CARD_THEMES_INSTALLER */
 
     /* Toolbar-only actions */
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
     { "LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL,
       G_CALLBACK (leave_fullscreen_cb) },
-#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
 #ifndef HAVE_HILDON
     { "ThemeMenu", NULL, N_("_Card Style"), NULL, NULL, NULL },
 #endif /* !HAVE_HILDON */
@@ -2338,12 +2304,10 @@ aisleriot_window_init (AisleriotWindow *window)
     "OptionsMenu",
     "Deal",
     "Hint",
+    "LeaveFullscreen",
 #ifdef HAVE_HILDON
     "AccelUndoMove",
 #endif /* HAVE_HILDON */
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
-    "LeaveFullscreen",
-#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
   };
 
   static const char ui_description[] =
@@ -2463,10 +2427,8 @@ aisleriot_window_init (AisleriotWindow *window)
         "<toolitem action='RedoMove'/>"
         "<toolitem action='Deal'/>"
         "<toolitem action='Hint'/>"
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
         "<separator name='LeaveFullscreenSep' expand='true'/>"
         "<toolitem action='LeaveFullscreen'/>"
-#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
 #ifdef ENABLE_DEBUG_UI
         "<toolitem action='DebugGameFirst'/>"
         "<toolitem action='DebugGamePrev'/>"
@@ -2487,7 +2449,7 @@ aisleriot_window_init (AisleriotWindow *window)
   GtkAccelGroup *accel_group;
   GtkAction *action;
   char *theme_name;
-  GamesCardTheme *theme;
+  ArCardTheme *theme;
   guint i;
 #ifndef HAVE_HILDON
   GtkStatusbar *statusbar;
@@ -2505,7 +2467,7 @@ aisleriot_window_init (AisleriotWindow *window)
 
   priv->game = aisleriot_game_new ();
 
-  priv->theme_manager = games_card_themes_new ();
+  priv->theme_manager = ar_card_themes_new ();
 
   priv->board_style = ar_style_new ();
 
@@ -2539,11 +2501,11 @@ aisleriot_window_init (AisleriotWindow *window)
 #endif /* HAVE_CLUTTER */
 
   theme_name = games_conf_get_string (NULL, aisleriot_conf_get_key (CONF_THEME), NULL);
-  theme = games_card_themes_get_theme_by_name (priv->theme_manager, theme_name);
+  theme = ar_card_themes_get_theme_by_name (priv->theme_manager, theme_name);
   g_free (theme_name);
   if (!theme) {
     /* Last-ditch fallback: try getting *any* theme */
-    theme = games_card_themes_get_theme_any (priv->theme_manager);
+    theme = ar_card_themes_get_theme_any (priv->theme_manager);
   }
   if (theme) {
     aisleriot_window_take_card_theme (window, theme /* adopts */);
@@ -2662,27 +2624,6 @@ aisleriot_window_init (AisleriotWindow *window)
   priv->main_menu = gtk_ui_manager_get_widget (priv->ui_manager, MAIN_MENU_UI_PATH);
   priv->toolbar = gtk_ui_manager_get_widget (priv->ui_manager, TOOLBAR_UI_PATH);
 
-#ifdef MAEMO_TOOLBAR_BANNER
-{
-  GtkToolItem *tool_item;
-
-  /* It's a bit ugly to mess with the toolbar like this,
-   * but it's simpler than to implement a dedicated action
-   * which creates a tool item which ellipsises its label...
-   */
-  tool_item = gtk_tool_item_new ();
-  gtk_tool_item_set_expand (tool_item, TRUE);
-
-  priv->game_message_label = GTK_LABEL (gtk_label_new (NULL));
-  gtk_misc_set_alignment (GTK_MISC (priv->game_message_label), 1.0, 0.5);
-  gtk_label_set_ellipsize (priv->game_message_label, PANGO_ELLIPSIZE_END);
-  gtk_container_add (GTK_CONTAINER (tool_item), GTK_WIDGET (priv->game_message_label));
-
-  gtk_toolbar_insert (GTK_TOOLBAR (priv->toolbar), tool_item, -1);
-  gtk_widget_show_all (GTK_WIDGET (tool_item));
-}
-#endif /* MAEMO_TOOLBAR_BANNER */
-
   /* Defer building the card themes menu until its parent's menu is opened */
 #ifdef HAVE_HILDON
   /* FIXMEchpe check this works! */
@@ -2707,7 +2648,7 @@ aisleriot_window_init (AisleriotWindow *window)
    * if the theme loading above involved the fallback); in that case
    * we need to update the menu right now.
    */
-  if (games_card_themes_get_themes_loaded (priv->theme_manager))
+  if (ar_card_themes_get_themes_loaded (priv->theme_manager))
     install_card_theme_menu (priv->theme_manager, window);
 
   /* Rebuild the themes menu when the themes list changes */
@@ -2748,7 +2689,7 @@ aisleriot_window_init (AisleriotWindow *window)
 
 #ifdef ENABLE_CARD_THEMES_INSTALLER
   action = gtk_action_group_get_action (priv->action_group, "InstallThemes");
-  gtk_action_set_sensitive (action, games_card_themes_can_install_themes (priv->theme_manager));
+  gtk_action_set_sensitive (action, ar_card_themes_can_install_themes (priv->theme_manager));
 #endif /* ENABLE_CARD_THEMES_INSTALLER */
 
   set_fullscreen_actions (window, FALSE);
