@@ -26,11 +26,6 @@ def gtkcolor_to_rgb (color):
             color.green / float(2**16),
             color.blue  / float(2**16))
 
-SPACING_FACTOR = 40 # The size of a box compared (roughly) to the size
-                    # of padding -- the larger this is, the smaller
-                    # the spaces
-SMALL_TO_BIG_FACTOR = 3.5 # The number of times wider than a small line a big line is.
-
 class SudokuNumberGrid (gtk.AspectFrame):
 
     def __init__ (self, group_size = 9):
@@ -50,32 +45,22 @@ class SudokuNumberGrid (gtk.AspectFrame):
         self.eb = gtk.EventBox()
         self.eb.add(self.table)
         self.add(self.eb)
-        self.connect('size-allocate', self.allocate_cb)
-        self.show_all()
-
-    def allocate_cb (self, w, rect):
-        if rect.width > rect.height:
-            side = rect.height
-        else: side = rect.width
-        # we want our small spacing to be 1/15th the size of a box
-        spacing = float(side) / (self.group_size * SPACING_FACTOR)
-        if spacing == 0:
-            spacing = 1
-        if hasattr(self, 'small_spacing') and spacing == self.small_spacing:
-            return
-        else:
-            self.change_spacing(spacing)
-
-    def change_spacing (self, small_spacing):
-        self.small_spacing = small_spacing
-        self.big_spacing = int(small_spacing*SMALL_TO_BIG_FACTOR)
-        self.table.set_row_spacings(int(small_spacing))
-        self.table.set_col_spacings(int(small_spacing))
+        self.table.set_row_spacings(1)
+        self.table.set_col_spacings(1)
         box_side = int(math.sqrt(self.group_size))
         for n in range(1, box_side):
-            self.table.set_row_spacing(box_side*n-1, self.big_spacing)
-            self.table.set_col_spacing(box_side*n-1, self.big_spacing)
-        self.table.set_border_width(self.big_spacing)
+            self.table.set_row_spacing(box_side*n-1, 2)
+            self.table.set_col_spacing(box_side*n-1, 2)
+        self.table.set_border_width(2)
+        self.show_all()
+
+    def set_parent_for(self, parent):
+        for entry in self.__entries__.values():
+            entry.set_parent_win(parent)
+
+    def set_timer(self, timer):
+        for entry in self.__entries__.values():
+            entry.set_timer(timer)
 
     def get_focused_entry (self):
         return self.table.focus_child
@@ -96,51 +81,6 @@ class SudokuNumberGrid (gtk.AspectFrame):
         for e in self.__entries__.values():
             e.modify_bg(gtk.STATE_NORMAL, color)
 
-class ParallelDict (dict):
-    """A handy new sort of dictionary for tracking conflicts.
-
-    pd = ParallelDict()
-    pd[1] = [2, 3, 4] # 1 is linked with 2, 3 and 4
-    pd -> {1:[2, 3, 4], 2:[1], 3:[1], 4:[1]}
-    pd[2] = [1, 3, 4] # 2 is linked with 3 and 4 as well as 1
-    pd -> {1: [2, 3, 4], 2:[3, 4], 3:[1, 2], 4:[1, 2]}
-    Now for the cool part...
-    del pd[1]
-    pd -> {2: [2, 3], 3:[2], 4:[2]}
-
-    Pretty neat, no?
-    """
-    def __init__ (self, *args):
-        dict.__init__(self, *args)
-
-    def __setitem__ (self, k, v):
-        dict.__setitem__(self, k, set(v))
-        for i in v:
-            if i == k:
-                continue
-            if self.has_key(i):
-                self[i].add(k)
-            else:
-                dict.__setitem__(self, i, set([k]))
-
-    def __delitem__ (self, k):
-        v = self[k]
-        dict.__delitem__(self, k)
-        for i in v:
-            if i == k:
-                continue
-            if self.has_key(i):
-                # Make sure we have a reference to i. If we don't
-                # something has gone wrong... but according to bug
-                # 385937 this has gone wrong at least once, so we'd
-                # better check for it.
-                if k in self[i]:
-                    self[i].remove(k)
-                if not self[i]:
-                    # If k was the last value in the list of values
-                    # for i, then we delete i from our dictionary
-                    dict.__delitem__(self, i)
-
 class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
 
     __gsignals__ = {
@@ -156,7 +96,6 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
         group_size = int(group_size)
         self.hints = 0
         self.always_show_hints = False
-        self.auto_fills = 0
         self.show_impossible_implications = show_impossible_implications
         self.impossible_hints = 0
         self.impossibilities = []
@@ -305,21 +244,40 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
                         self.remove(x, y, do_removal = True)
         return removed
 
-    def clear_notes (self, clear_args = {'top_text':'', 'bottom_text':''}):
-        """Remove all notes."""
-        self.removed = []
+    def clear_notes (self, side = 'Both'):
+        '''Remove notes
+
+        The list of notes removed by this function are returned in a list.
+        The side argument determines what notes get cleared as well as what
+        notes get returned.
+        'Both' - Clears both the top and bottom notes
+        'Top' - Clear only the top notes
+        'Bottom' - Clear only the bottom notes
+        '''
+        # Set the argument list for NumberBox.set_note_text()
+        if side == 'Both':
+            clear_args = {'top_text':'', 'bottom_text':''}
+        elif side == 'Top':
+            clear_args = {'top_text':''}
+        else:
+            clear_args = {'bottom_text':''}
+        # Storage for removed notes
+        removed = []
         for x in range(self.group_size):
             for y in range(self.group_size):
                 e = self.__entries__[(x, y)]
                 top, bottom = e.get_note_text()
+                # Don't return the bottom notes if we're only clearing the top
+                # or the top notes if we're only clearing the bottom.
+                if side == 'Top':
+                    bottom = ''
+                elif side == 'Bottom':
+                    top = ''
                 if top or bottom:
-                    self.removed.append((x, y, (top, bottom)))
+                    removed.append((x, y, (top, bottom)))
                     e.set_note_text(**clear_args)
                     e.queue_draw()
-        return self.removed
-
-    def clear_hints (self):
-        self.clear_notes(clear_args = {'bottom_text':''})
+        return removed
 
     @simple_debug
     def blank_grid (self):
@@ -329,12 +287,14 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
                 if e.get_value():
                     self.remove(x, y)
                 e.set_read_only(False)
+        for imp_cell in self.impossibilities:
+            self.__entries__[imp_cell].set_text('')
+        self.impossibilities = []
         self.grid = None
         self.clear_notes()
 
     @simple_debug
     def change_grid (self, grid, group_size):
-        self.auto_fills = 0
         self.hints = 0
         self.impossible_hints = 0
         self.trackers = {}
@@ -368,7 +328,6 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
     @simple_debug
     def setup_grid (self, grid, group_size):
         self.doing_initial_setup = True
-        self.__error_pairs__ = ParallelDict()
         if isinstance(grid, sudoku.SudokuGrid):
             self.grid = sudoku.InteractiveSudoku(grid.grid, group_size = grid.group_size)
         else:
@@ -388,10 +347,6 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
             self.remove(widget.x, widget.y)
         else:
             self.entry_validate(widget)
-        if self.show_impossible_implications:
-            self.mark_impossible_implications(widget.x, widget.y)
-        if self.always_show_hints:
-            self.update_all_hints()
 
     def update_all_hints (self):
         for x in range(self.group_size):
@@ -411,16 +366,21 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
         if self.grid.check_for_completeness():
             self.emit('puzzle-finished')
 
-    def complain_conflicts (self, x, y, value):
-        '''set error highlights on [x, y] and all related box.
+    def highlight_conflicts (self, x, y):
+        '''highlight any squares that conflict with position x,y.
 
-        We think the error is caused by `value`. But the box at [x, y] could
-        have a different value.'''
-        self.__entries__[x, y].set_error_highlight(True)
-        conflicts = self.grid.find_conflicts(x, y, value)
-        for conflict in conflicts:
-            self.__entries__[conflict].set_error_highlight(True)
-        self.__error_pairs__[(x, y)] = conflicts
+        Conflict resolution is taken care of completely within
+        the InteractiveGrid class.  A list of conflicting cells
+        are stored in InteractiveGrid.conflicts
+        '''
+        # Return if there are no conflicts for this cell
+        if not self.grid.conflicts.has_key((x, y)):
+            return
+        # Highlight the current cell
+        self.__entries__[(x, y)].set_error_highlight(True)
+        # Then highlight any cells that conflict with it
+        for coord in self.grid.conflicts[(x, y)]:
+            self.__entries__[coord].set_error_highlight(True)
 
     @simple_debug
     def add_value (self, x, y, val, trackers = []):
@@ -452,15 +412,17 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
                 if v:
                     self.__entries__[(x, y)].set_color(self.get_tracker_color(k))
                     self.trackers[k].append((x, y, val))
-        # Add value to our underlying sudoku grid -- this will raise
-        # an error if the value is out of bounds with the current
-        # rules.
-        try:
-            self.grid.add(x, y, val, True)
-        except sudoku.ConflictError, err:
-            self.complain_conflicts(err.x, err.y, err.value)
+        # Add it to the underlying grid
+        self.grid.add(x, y, val, True)
+        # Highlight any conflicts that the new value creates
+        self.highlight_conflicts(x, y)
         # Draw our entry
         self.__entries__[(x, y)].queue_draw()
+        # Update all hints if we need to
+        if self.always_show_hints and not self.doing_initial_setup:
+            self.update_all_hints()
+        if not self.doing_initial_setup:
+            self.mark_impossible_implications(x, y)
 
     @simple_debug
     def remove (self, x, y, do_removal = False):
@@ -469,9 +431,10 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
         If do_removal, remove it from our underlying grid as well.
         """
         e = self.__entries__[(x, y)]
-        if do_removal and self.grid and self.grid._get_(x, y):
+        # Always call the grid's remove() for proper conflict resolution
+        if self.grid:
             self.grid.remove(x, y)
-        self.remove_error_highlight(x, y)
+            self.remove_error_highlight()
         # remove trackers
         for t in self.trackers_for_point(x, y):
             remove = []
@@ -483,19 +446,24 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
         if e.get_text():
             e.set_value(0)
         e.unset_color()
+        # Update all hints if we need to
+        if self.grid and self.always_show_hints and not self.doing_initial_setup:
+            self.update_all_hints()
+        if not self.doing_initial_setup:
+            self.mark_impossible_implications(x, y)
 
-    def remove_error_highlight (self, x, y):
-        '''remove error highlight from [x, y] and also all errors caused by it'''
-        if self.__error_pairs__.has_key((x, y)):
-            entry = self.__entries__[(x, y)]
-            entry.set_error_highlight(False)
-            errors_removed = self.__error_pairs__[(x, y)]
-            del self.__error_pairs__[(x, y)]
-            for coord in errors_removed:
-                # If we're not an error by some other pairing...
-                if not self.__error_pairs__.has_key(coord):
-                    linked_entry = self.__entries__[coord]
-                    linked_entry.set_error_highlight(False)
+    def remove_error_highlight (self):
+        '''remove error highlight from [x, y] and also all errors caused by it
+
+        Conflict resolution is now handled within the InteractiveSudoku class.
+        If any conflicts were cleared on the last remove() then they are
+        stored in grid.cleared_conflicts
+        '''
+        if not self.grid.cleared_conflicts:
+            return
+        for coord in self.grid.cleared_conflicts:
+            linked_entry = self.__entries__[coord]
+            linked_entry.set_error_highlight(False)
 
     @simple_debug
     def auto_fill (self):
@@ -504,10 +472,6 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
         for coords, val in changed:
             self.add_value(coords[0], coords[1], val)
             retval.append((coords[0], coords[1], val))
-            if self.show_impossible_implications:
-                self.mark_impossible_implications(*coords)
-        if retval:
-            self.auto_fills += 1
         if self.grid.check_for_completeness():
             self.emit('puzzle-finished')
         return retval
@@ -522,20 +486,92 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
             e.set_text_interactive('')
             e.set_text_interactive(str(filled[1]))
 
+    def __set_impossible (self, coords, setting):
+        '''Call set_impossible() on a grid entry
+
+        This function is a helper for the 'Warn about unfillable squares'
+        feature.  It only calls set_impossible() if the option is on to prevent
+        a check against the show_impossible_implications flag elsewhere.  The
+        return from this function indicates whether or not the cell "may"
+        have been modified, which is basically the value of the option setting.
+        '''
+        if self.show_impossible_implications:
+            self.__entries__[coords].set_impossible(setting)
+        return self.show_impossible_implications
+
+    def display_impossible_implications (self):
+        '''Start X-marking cells that have no possible values
+        '''
+        self.show_impossible_implications = True
+        for imp_cell in self.impossibilities:
+            self.__set_impossible(imp_cell, True)
+            self.impossible_hints += 1
+        if self.always_show_hints:
+            self.update_all_hints()
+
+    def hide_impossible_implications (self):
+        '''Stop X-marking cells that have no possible values
+        '''
+        for imp_cell in self.impossibilities:
+            self.__set_impossible(imp_cell, False)
+        self.show_impossible_implications = False
+        if self.always_show_hints:
+            self.update_all_hints()
+
     @simple_debug
-    def mark_impossible_implications (self, x, y):
+    def mark_impossible_implications (self, x, y, check_conflicts = True):
+        '''Mark cells with X if they have no possible values
+
+        The hint this method provides can be turned on and off from the
+        menu Tools->'Warn about unfillable squares' option.
+
+        The check_conflicts parameter is for internal use only.  It is used as
+        a one level recursion on conflicts that the original target is involved
+        with.
+
+        Impossibilities are tracked regardless of the user's option setting.
+        This was done to allow the user to toggle the option mid-game and still
+        behave properly.  Conditional X-marking of cells happens in the
+        __set_impossible() function.
+        '''
+        # Make sure we have a grid to work with
         if not self.grid:
             return
+        # Flag whether or not we need to update hints
+        grid_modified = False
+        # Find any new impossible cells based on calling cell
         implications = self.grid.find_impossible_implications(x, y)
         if implications:
-            for x, y in implications:
-                self.__entries__[(x, y)].set_impossible(True)
-                if not (x, y) in self.impossibilities:
-                    self.impossible_hints += 1
-        for x, y in self.impossibilities:
-            if not (x, y) in implications:
-                self.__entries__[(x, y)].set_impossible(False)
-        self.impossibilities = implications
+            for imp_cell in implications:
+                grid_modified = self.__set_impossible(imp_cell, True)
+                # Add them to the list if they aren't there already...
+                if not imp_cell in self.impossibilities:
+                    self.impossibilities.append(imp_cell)
+                    # But don't score it unless the option is on
+                    if self.show_impossible_implications:
+                        self.impossible_hints += 1
+        # Reset the list of impossible cells ignoring the called cell. Use a
+        # copy to iterate over, so items can be removed while looping.
+        if self.impossibilities:
+            for imp_cell in self.impossibilities[:]:
+                if imp_cell == (x, y):
+                    continue
+                if self.grid.possible_values(*imp_cell):
+                    self.impossibilities.remove(imp_cell)
+                    grid_modified = self.__set_impossible(imp_cell, False)
+                else:
+                    grid_modified = self.__set_impossible(imp_cell, True)
+        # If any conflicts have been cleared or created, mark any impossible
+        # cells they may have caused or removed
+        if check_conflicts:
+            for xx, yy in self.grid.cleared_conflicts:
+                self.mark_impossible_implications(xx, yy, False)
+            if self.grid.conflicts.has_key((x, y)):
+                for xx, yy in self.grid.conflicts[(x, y)]:
+                    self.mark_impossible_implications(xx, yy, False)
+        # Update the hints if we need to
+        if grid_modified and self.always_show_hints:
+            self.update_all_hints()
 
     @simple_debug
     def create_tracker (self, identifier = 0):
@@ -608,6 +644,9 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
 
     def add_tracker (self, x, y, tracker, val = None):
         self.__entries__[(x, y)].set_color(self.get_tracker_color(tracker))
+        # Highlight the conflicts when opening a saved game
+        if self.grid.conflicts.has_key((x, y)):
+            self.__entries__[(x, y)].set_error_highlight(True)
         if not val:
             val = self.grid._get_(x, y)
         self.trackers[tracker].append((x, y, val))
