@@ -18,7 +18,6 @@ import game_selector
 import gsudoku
 import printing
 import saver
-import sudoku_generator_gui
 import sudoku_maker
 import timer
 from defaults import (APPNAME, APPNAME_SHORT, AUTHORS, COPYRIGHT, DESCRIPTION, DOMAIN, 
@@ -47,18 +46,18 @@ def inactivate_new_game_etc (fun):
     def inactivate_new_game_etc_ (ui, *args, **kwargs):
         paths = [
             '/MenuBar/Game/New',
+            '/MenuBar/Game/Reset',
+            '/MenuBar/Game/PuzzleInfo',
             '/MenuBar/Game/Print',
             # undo/redo is handled elsewhere as it can't simply be turned on/off.
-            '/MenuBar/Edit/Clear',
-            '/MenuBar/Edit/ClearNotes',
-            '/MenuBar/View/ToggleToolbar',
+            '/MenuBar/Settings/ToggleToolbar',
+            '/MenuBar/Settings/ToggleHighlight',
+            '/MenuBar/Settings/AlwaysShowPossible',
+            '/MenuBar/Settings/ShowImpossibleImplications',
             '/MenuBar/Tools/ShowPossible',
-            '/MenuBar/Tools/AutofillCurrentSquare',
-            '/MenuBar/Tools/Autofill',
-            '/MenuBar/Tools/AlwaysShowPossible',
-            '/MenuBar/Tools/ShowImpossibleImplications',
+            '/MenuBar/Tools/ClearTopNotes',
+            '/MenuBar/Tools/ClearBottomNotes',
             '/MenuBar/Tools/Tracker',
-            '/MenuBar/Game/PuzzleInfo',
             ]
         for p in paths:
             action = ui.uimanager.get_action(p)
@@ -85,6 +84,10 @@ class UI (gconf_wrapper.GConfWrapper):
     <menubar name="MenuBar">
       <menu name="Game" action="Game">
         <menuitem action="New"/>
+        <menuitem action="Reset"/>
+        <separator/>
+        <menuitem action="Undo"/>
+        <menuitem action="Redo"/>
         <separator/>
         <menuitem action="PuzzleInfo"/>
         <separator/>
@@ -93,32 +96,22 @@ class UI (gconf_wrapper.GConfWrapper):
         <separator/>
         <menuitem action="Close"/>
       </menu>
-      <menu action="Edit">
-        <menuitem action="Undo"/>
-        <menuitem action="Redo"/>
-        <separator/>
-        <menuitem action="Clear"/>
-        <menuitem action="ClearNotes"/>
-      </menu>
-      <menu action="View">
+      <menu action="Settings">
         <menuitem action="FullScreen"/>
-        <separator/>
         <menuitem action="ToggleToolbar"/>
+        <separator/>
         <menuitem action="ToggleHighlight"/>
+        <menuitem action="AlwaysShowPossible"/>
+        <menuitem action="ShowImpossibleImplications"/>
       </menu>
       <menu action="Tools">
         <menuitem action="ShowPossible"/>
-        <menuitem action="AutofillCurrentSquare"/>
-        <menuitem action="Autofill"/>
         <separator/>
-        <menuitem action="AlwaysShowPossible"/>
-        <menuitem action="ShowImpossibleImplications"/>
-        <separator/>
-        <menuitem action="Generator"/>
-        <menuitem action="BackgroundGenerator"/>
+        <menuitem action="ClearTopNotes"/>
+        <menuitem action="ClearBottomNotes"/>
         <separator/>
         <menuitem action="Tracker"/>
-        </menu>
+      </menu>
       <menu action="Help">
         <menuitem action="ShowHelp"/>
         <menuitem action="About"/>
@@ -126,15 +119,12 @@ class UI (gconf_wrapper.GConfWrapper):
     </menubar>
     <toolbar name="Toolbar">
       <toolitem action="New"/>
-      <toolitem action="Print"/>
       <separator/>
       <toolitem action="Undo"/>
       <toolitem action="Redo"/>
       <separator/>
       <toolitem action="ShowPossible"/>
-      <toolitem action="AutofillCurrentSquare"/>
       <separator/>
-      <toolitem action="ToggleHighlight"/>
       <toolitem action="Tracker"/>
     </toolbar>
     </ui>'''
@@ -164,6 +154,7 @@ class UI (gconf_wrapper.GConfWrapper):
                                             )
         self.setup_gui()
         self.timer = timer.ActiveTimer(self.w)
+        self.gsd.set_timer(self.timer)
         self.won = False
         # add the accelerator group to our toplevel window
         self.worker_connections = []
@@ -181,25 +172,21 @@ class UI (gconf_wrapper.GConfWrapper):
                 self.quit = True
             else:
                 self.quit = False
-                # Generate puzzles in background...
-                if self.gconf['generate_puzzles_in_background']:
-                    gobject.timeout_add_seconds(1, lambda *args: self.start_worker_thread() and True)
 
+        # Generate puzzles in background...
+        gobject.timeout_add_seconds(1, lambda *args: self.start_worker_thread() and True)
 
     @inactivate_new_game_etc
     def select_game (self):
         self.tb.hide()
-        self.update_statusbar()
         choice = game_selector.NewOrSavedGameSelector().run_swallowed_dialog(self.swallower)
         if not choice:
             return True
         self.timer.start_timing()
         if choice[0] == game_selector.NewOrSavedGameSelector.NEW_GAME:
             self.gsd.change_grid(choice[1], 9)
-            self.update_statusbar()
         if choice[0] == game_selector.NewOrSavedGameSelector.SAVED_GAME:
             saver.open_game(self, choice[1])
-            self.update_statusbar()
         if self.gconf['show_toolbar']:
             self.tb.show()
         if self.gconf['always_show_hints']:
@@ -216,6 +203,7 @@ class UI (gconf_wrapper.GConfWrapper):
         self.initialize_prefs()
         self.setup_main_window()
         self.gsd = gsudoku.SudokuGameDisplay()
+        self.gsd.set_parent_for(self.w)
         self.gsd.connect('puzzle-finished', self.you_win_callback)
         self.setup_color()
         self.setup_actions()
@@ -239,84 +227,68 @@ class UI (gconf_wrapper.GConfWrapper):
         self.main_actions = gtk.ActionGroup('MainActions')
         self.main_actions.add_actions([
             ('Game', None, _('_Game')),
-            ('New', gtk.STOCK_NEW, None,
-             '<Control>n', _('New game'), self.new_cb),
-            ('Print', gtk.STOCK_PRINT, None,
-             None, _('Print current game'), self.print_game),
-            ('PrintMany', gtk.STOCK_PRINT, _('Print _Multiple Sudokus'),
-             None, _('Print more than one sudoku at a time.'), self.print_multiple_games),
-            ('Close', gtk.STOCK_CLOSE, None, '<Control>w',
-             _('Close Sudoku'), self.quit_cb),
+            ('New', gtk.STOCK_NEW, None, '<Control>n', _('New game'), self.new_cb),
+            ('Reset', gtk.STOCK_CLEAR, _('_Reset'), '<Control>b',
+             _("Reset current grid(do-over)"), self.game_reset_cb),
+            ('Undo', gtk.STOCK_UNDO, _('_Undo'), '<Control>z',
+             _('Undo last action'), self.stop_dancer),
+            ('Redo', gtk.STOCK_REDO, _('_Redo'), '<Shift><Control>z',
+             _('Redo last action')),
+            ('PuzzleInfo', gtk.STOCK_ABOUT, _('Puzzle _Statistics...'), None,
+             _('Show statistics about current puzzle'), self.show_info_cb),
+            ('Print', gtk.STOCK_PRINT, _('_Print...'), '<Control>p', _('Print current game'), self.print_game),
+            ('PrintMany', gtk.STOCK_PRINT, _('Print _Multiple Sudokus...'), None,
+             _('Print more than one sudoku at a time.'), self.print_multiple_games),
+            ('Close', gtk.STOCK_CLOSE, None, '<Control>w', _('Close Sudoku'), self.quit_cb),
+            ('Settings', None, _('_Settings')),
+            ('FullScreen', gtk.STOCK_FULLSCREEN, None, 'F11', None, self.full_screen_cb),
             ('Tools', None, _('_Tools')),
-            ('View', None, _('_View')),
-            ('ShowPossible', gtk.STOCK_DIALOG_INFO, _('_Hint'),
-             '<Control>h',
-             _('Show which numbers could go in the current square.'),
-             self.show_hint_cb),
-            ('AutofillCurrentSquare', gtk.STOCK_APPLY, _('_Fill'), '<Control>f',
-             _('Automatically fill in the current square if possible.'),
-             self.auto_fill_current_square_cb),
-            ('Autofill', gtk.STOCK_REFRESH, _('Fill _all squares'), '<Control>a',
-             _('Automatically fill in all squares for which there is only one valid value.'),
-             self.auto_fill_cb),
-            ('FullScreen', gtk.STOCK_FULLSCREEN, None,
-             'F11', None, self.full_screen_cb),
-            ('Generator', None, _('_Generate new puzzles'), None, _('Generate new puzzles.'),
-              self.generate_puzzle_gui, ),
-            ('PuzzleInfo', gtk.STOCK_ABOUT, _('Puzzle _Statistics'),
-             None, _('Show statistics about current puzzle'),
-             self.show_info_cb),
-            ('Help', None, _('_Help'),
-             None, None, None),
-            ('About', gtk.STOCK_ABOUT, None,
-             None, None, self.show_about),
-            ('ShowHelp', gtk.STOCK_HELP, _('_Contents'),
-             'F1', None, self.show_help),
+            ('ShowPossible', gtk.STOCK_DIALOG_INFO, _('_Hint'), '<Control>h',
+             _('Show which numbers could go in the current square.'), self.show_hint_cb),
+            ('ClearTopNotes', None, _('Clear _Top Notes'), '<Control>j',
+             _("Clear all of the top notes"), self.clear_top_notes_cb),
+            ('ClearBottomNotes', None, _('Clear _Bottom Notes'), '<Control>k',
+             _("Clear all of the bottom notes"), self.clear_bottom_notes_cb),
+            ('Help', None, _('_Help'), None, None, None),
+            ('ShowHelp', gtk.STOCK_HELP, _('_Contents'), 'F1', None, self.show_help),
+            ('About', gtk.STOCK_ABOUT, None, None, None, self.show_about),
             ])
         self.main_actions.add_toggle_actions([
             ('AlwaysShowPossible',
              None,
-             _('_Always show hint'),
+             _('Show _Possible Numbers'),
              None,
              _('Always show possible numbers in a square'),
              self.auto_hint_cb),
             ('ShowImpossibleImplications',
              None,
-             _('Warn about _unfillable squares'),
+             _('Warn About _Unfillable Squares'),
              None,
              _('Warn about squares made unfillable by a move'),
              self.impossible_implication_cb),
-            ('Tracker', 'tracks', _('_Track additions'),
+            ('Tracker', 'tracks', _('_Track Additions'),
              '<Control>T',
              _('Mark new additions in a separate color so you can keep track of them.'),
              self.tracker_toggle_cb, False),
             ('ToggleToolbar', None, _('Show _Toolbar'), None, None, self.toggle_toolbar_cb, True),
             ('ToggleHighlight', gtk.STOCK_SELECT_COLOR, _('_Highlighter'),
-             None, _('Highlight the current row, column and box'), self.toggle_highlight_cb, False),
-            ('BackgroundGenerator', None, _('Generate new puzzles _while you play'),
-             None,
-             _('Generate new puzzles in the background while you play. This will automatically pause when the game goes into the background.'),
-             self.toggle_generator_cb, True),
+             None, _('Highlight the current row, column and box'), self.toggle_highlight_cb, False)
             ])
 
-        self.edit_actions = gtk.ActionGroup('EditActions')
-        self.edit_actions.add_actions(
-            [('Edit', None, _('_Edit')),
-             ('Undo', gtk.STOCK_UNDO, _('_Undo'), '<Control>z', _('Undo last action'), self.stop_dancer),
-             ('Redo', gtk.STOCK_REDO, _('_Redo'), '<Shift><Control>z', _('Redo last action')),
-             ('Clear', gtk.STOCK_CLEAR, _('_Clear'), '<Control>b', _("Clear entries you've filled in"), self.clear_cb),
-             ('ClearNotes', None, _('Clear _Notes'), None, _("Clear notes and hints"), self.clear_notes_cb),
-             ])
+        self.main_actions.get_action('Undo').set_is_important(True)
+        self.main_actions.get_action('Redo').set_is_important(True)
+        self.main_actions.get_action('ShowPossible').set_is_important(True)
+        self.main_actions.get_action('Tracker').set_is_important(True)
+
         self.uimanager.insert_action_group(self.main_actions, 0)
-        self.uimanager.insert_action_group(self.edit_actions, 0)
         self.uimanager.add_ui_from_string(self.ui)
 
     def setup_undo (self):
         self.cleared = [] # used for Undo memory
         self.cleared_notes = [] # used for Undo memory
         # Set up our UNDO stuff
-        undo_widg = self.edit_actions.get_action('Undo')
-        redo_widg = self.edit_actions.get_action('Redo')
+        undo_widg = self.main_actions.get_action('Undo')
+        redo_widg = self.main_actions.get_action('Redo')
         self.history = Undo.UndoHistoryList(undo_widg, redo_widg)
         for entry in self.gsd.__entries__.values():
             Undo.UndoableGenericWidget(entry, self.history,
@@ -359,9 +331,6 @@ class UI (gconf_wrapper.GConfWrapper):
         self.vb.show()
         self.game_box.show()
         self.main_area.pack_start(self.game_box, False, padding = 12)
-        self.statusbar = gtk.Statusbar()
-        self.statusbar.show()
-        self.vb.pack_end(self.statusbar, fill = False, expand = False)
         self.w.add(self.vb)
 
     def setup_toggles (self):
@@ -371,8 +340,6 @@ class UI (gconf_wrapper.GConfWrapper):
               self.main_actions.get_action('AlwaysShowPossible')),
              ('show_impossible_implications',
               self.main_actions.get_action('ShowImpossibleImplications')),
-             ('generate_puzzles_in_background',
-              self.main_actions.get_action('BackgroundGenerator')),
              ('show_toolbar',
               self.main_actions.get_action('ToggleToolbar')),
              ('highlight',
@@ -384,13 +351,16 @@ class UI (gconf_wrapper.GConfWrapper):
     @simple_debug
     def start_worker_thread (self, *args):
         n_new_puzzles = self.sudoku_maker.n_puzzles(new = True)
-        if n_new_puzzles < self.gconf['minimum_number_of_new_puzzles']:
-            self.worker = threading.Thread(target = lambda *args: self.sudoku_maker.work(limit = 5))
-            self.worker_connections = [
-                self.timer.connect('timing-started', self.sudoku_maker.resume),
-                self.timer.connect('timing-stopped', self.sudoku_maker.pause)
-                ]
-            self.worker.start()
+        try:
+            if n_new_puzzles < self.gconf['minimum_number_of_new_puzzles']:
+                self.worker = threading.Thread(target = lambda *args: self.sudoku_maker.work(limit = 5))
+                self.worker_connections = [
+                    self.timer.connect('timing-started', self.sudoku_maker.resume),
+                    self.timer.connect('timing-stopped', self.sudoku_maker.pause)
+                    ]
+                self.worker.start()
+        except gconf_wrapper.GConfError:
+            pass # assume we have enough new puzzles
         return True
 
     @simple_debug
@@ -405,6 +375,11 @@ class UI (gconf_wrapper.GConfWrapper):
             self.dancer.stop_dancing()
             delattr(self, 'dancer')
 
+    def start_dancer (self):
+        import dancer
+        self.dancer = dancer.GridDancer(self.gsd)
+        self.dancer.start_dancing()
+
     @simple_debug
     def you_win_callback (self, grid):
         if hasattr(self, 'dancer'):
@@ -414,24 +389,21 @@ class UI (gconf_wrapper.GConfWrapper):
         self.gconf['difficulty'] = self.gconf['difficulty'] + 0.1
         self.timer.finish_timing()
         self.sudoku_tracker.finish_game(self)
-        sublabel = _("You completed the puzzle in %(totalTime)s (%(activeTime)s active)") % {'totalTime': self.timer.total_time_string(),
-        'activeTime': self.timer.active_time_string()
-                }
+        if self.timer.active_time != self.timer.total_time:
+            sublabel = _("You completed the puzzle in %(totalTime)s (%(activeTime)s active).") % {'totalTime': self.timer.total_time_string(),
+            'activeTime': self.timer.active_time_string()
+                    }
+        else:
+            sublabel = _("You completed the puzzle in %(totalTime)s.") % {'totalTime': self.timer.total_time_string()}
         sublabel += "\n"
-        sublabel += ngettext("You got %(n)s hint", "You got %(n)s hints", self.gsd.hints) % {'n':self.gsd.hints}
+        sublabel += ngettext("You got %(n)s hint.", "You got %(n)s hints.", self.gsd.hints) % {'n':self.gsd.hints}
         sublabel += "\n"
         if self.gsd.impossible_hints:
             sublabel += ngettext("You had %(n)s impossibility pointed out.",
                                  "You had %(n)s impossibilities pointed out.",
                                  self.gsd.impossible_hints) % {'n':self.gsd.impossible_hints}
             sublabel += "\n"
-        if self.gsd.auto_fills:
-            sublabel += ngettext("You used the auto-fill %(n)s time",
-                                 "You used the auto-fill %(n)s times",
-                                 self.gsd.auto_fills) % {'n':self.gsd.auto_fills}
-        import dancer
-        self.dancer = dancer.GridDancer(self.gsd)
-        self.dancer.start_dancing()
+        self.start_dancer()
         dialog_extras.show_message(_("You win!"), label = _("You win!"),
                                    sublabel = sublabel
                                    )
@@ -532,36 +504,82 @@ class UI (gconf_wrapper.GConfWrapper):
             self.is_fullscreen = True
 
     @simple_debug
-    def clear_cb (self, *args):
+    def game_reset_cb (self, *args):
         clearer = Undo.UndoableObject(
-            self.do_clear, #action
-            self.undo_clear, #inverse
+            self.do_game_reset, #action
+            self.undo_game_reset, #inverse
             self.history #history
             )
         clearer.perform()
 
-    # add a check to stop the dancer if she is dancing
-    def do_clear (self, *args):
+    def do_game_reset (self, *args):
         self.cleared.append(self.gsd.reset_grid())
         self.stop_dancer()
+        self.do_clear_notes()
 
-    # add a check for finish in the undo to clear
-    def undo_clear (self, *args):
+    def undo_game_reset (self, *args):
         for entry in self.cleared.pop():
             self.gsd.add_value(*entry)
-        if self.gsd.grid.check_for_completeness():
-            self.gsd.emit('puzzle-finished')
+        self.undo_clear_notes()
 
-    def clear_notes_cb (self, *args):
+    def clear_top_notes_cb (self, *args):
         clearer = Undo.UndoableObject(
-            lambda *args: self.cleared_notes.append(self.gsd.clear_notes()), #action
-            # clear_notes returns a list of tuples indicating the cleared notes...
-            # (x,y,(top,bottom)) -- this is what we need for undoing
-            lambda *args: [self.gsd.__entries__[t[0], t[1]].set_notes(t[2]) for t in self.cleared_notes.pop()], #inverse
+            lambda *args: self.do_clear_notes('Top'), #action
+            self.undo_clear_notes, #inverse
             self.history
             )
         clearer.perform()
 
+    def clear_bottom_notes_cb (self, *args):
+        clearer = Undo.UndoableObject(
+            lambda *args: self.do_clear_notes('Bottom'), #action
+            self.undo_clear_notes, #inverse
+            self.history
+            )
+        clearer.perform()
+
+    def do_clear_notes(self, side = 'Both'):
+        ''' Clear top, bottom, or all notes - in undoable fashion
+
+        The side argument is used to specify which notes
+        are to be cleared.
+        'Top' - just clear the top notes
+        'Bottom' - just clear the bottom notes
+        'Both' - clear all notes(argument default)
+
+        Store all of the cleared notes in the cleared_notes list so the undo
+        can pick up on them later.  The list items are in the format
+        (x, y, (top note, bottom note)).
+        '''
+        self.cleared_notes.append(self.gsd.clear_notes(side))
+        # Turn off auto-hint if the player clears the bottom notes
+        if side == 'Bottom' and self.gconf['always_show_hints']:
+            always_show_hint_wdgt = self.main_actions.get_action('AlwaysShowPossible')
+            always_show_hint_wdgt.activate()
+        # Update the hints...in case we're redoing a clear of them
+        if self.gconf['always_show_hints']:
+            self.gsd.update_all_hints()
+
+    def undo_clear_notes(self):
+        ''' Undo previously cleared notes
+
+        Clearing notes sets the cleared_notes list of tuples indicating the
+        notes that were cleared. They are in the format
+        (x, y, (top note ,bottom note))
+        '''
+        for x, y, notes in self.cleared_notes.pop():
+            top, bottom = notes
+            if top:
+                self.gsd.__entries__[x, y].set_note_text(top_text = top)
+            if bottom:
+                self.gsd.__entries__[x, y].set_note_text(bottom_text = bottom)
+        # Update the hints...in case we're undoing over top of them
+        if self.gconf['always_show_hints']:
+            self.gsd.update_all_hints()
+        # Make sure we're still dancing if we undo after win
+        if self.gsd.grid.check_for_completeness():
+            self.start_dancer()
+
     @simple_debug
     def show_hint_cb (self, *args):
         self.gsd.show_hint()
@@ -573,41 +591,14 @@ class UI (gconf_wrapper.GConfWrapper):
             self.gsd.update_all_hints()
         else:
             self.gsd.always_show_hints = False
-            self.gsd.clear_hints()
+            self.gsd.clear_notes('Bottom')
 
     @simple_debug
     def impossible_implication_cb (self, action):
         if action.get_active():
-            self.gsd.show_impossible_implications = True
+            self.gsd.display_impossible_implications()
         else:
-            self.gsd.show_impossible_implications = False
-
-    @simple_debug
-    def auto_fill_cb (self, *args):
-        if not hasattr(self, 'autofilled'):
-            self.autofilled = []
-        if not hasattr(self, 'autofiller'):
-            self.autofiller = Undo.UndoableObject(
-                self.do_auto_fill,
-                self.undo_auto_fill,
-                self.history
-                )
-        self.autofiller.perform()
-
-    def do_auto_fill (self, *args):
-        self.autofilled.append(self.gsd.auto_fill())
-        if self.gconf['always_show_hints']:
-            self.gsd.update_all_hints()
-
-    def undo_auto_fill (self, *args):
-        for entry in self.autofilled.pop():
-            self.gsd.remove(entry[0], entry[1], do_removal = True)
-        if self.gconf['always_show_hints']:
-            self.gsd.update_all_hints()
-
-    @simple_debug
-    def auto_fill_current_square_cb (self, *args):
-        self.gsd.auto_fill_current_entry()
+            self.gsd.hide_impossible_implications()
 
     @simple_debug
     def setup_tracker_interface (self):
@@ -631,27 +622,6 @@ class UI (gconf_wrapper.GConfWrapper):
         else:
             self.tb.hide()
 
-    def set_statusbar_value (self, status):
-        if not hasattr(self, 'sbid'):
-            self.sbid = self.statusbar.get_context_id('game_info')
-        self.statusbar.pop(self.sbid)
-        self.statusbar.push(self.sbid, status)
-
-
-    def update_statusbar (self, *args):
-        if not self.gsd.grid:
-            self.set_statusbar_value(" ")
-            return True
-
-        puzzle = self.gsd.grid.virgin.to_string()
-        puzzle_diff = self.sudoku_maker.get_difficulty(puzzle)
-
-        tot_string = _("Playing %(difficulty)s puzzle.") % {'difficulty':puzzle_diff.value_string()}
-        tot_string += " " + "(%1.2f)" % puzzle_diff.value
-
-        self.set_statusbar_value(tot_string)
-        return True
-
     def toggle_highlight_cb (self, widg):
         if widg.get_active():
             self.gsd.toggle_highlight(True)
@@ -685,13 +655,6 @@ class UI (gconf_wrapper.GConfWrapper):
                                    label = _("Puzzle Statistics"),
                                    sublabel = information)
 
-    @simple_debug
-    def toggle_generator_cb (self, toggle):
-        if toggle.get_active():
-            self.start_worker_thread()
-        else:
-            self.stop_worker_thread()
-
     @simple_debug
     def autosave (self):
         # this is called on a regular loop and will autosave if we
@@ -735,10 +698,6 @@ class UI (gconf_wrapper.GConfWrapper):
         gp = printing.GamePrinter(self.sudoku_maker, self.gconf)
         gp.run_dialog()
 
-    @simple_debug
-    def generate_puzzle_gui (self, *args):
-        sudoku_generator_gui.GameGenerator(self, self.gconf)
-
 class TrackerBox (gtk.VBox):
 
     @simple_debug
@@ -786,18 +745,10 @@ class TrackerBox (gtk.VBox):
               None, _('Clear all moves tracked by selected tracker.'),
               self.clear_cb
               ),
-             ('Keep', None,
-              _('_Clear Others'),
-              None,
-              _('Clear all moves not tracked by selected tracker.'),
-              self.keep_cb),
              ]
             )
-        for action, widget_name in [('Clear', 'ClearTrackerButton'),
-                                   ('Keep', 'KeepTrackerButton'),
-                                   ]:
-            a = self.tracker_actions.get_action(action)
-            a.connect_proxy(self.builder.get_object(widget_name))
+        a = self.tracker_actions.get_action('Clear')
+        a.connect_proxy(self.builder.get_object('ClearTrackerButton'))
         self.builder.get_object('AddTrackerButton').connect('clicked',
                                                           self.add_tracker)
         # Default to insensitive (they only become sensitive once a tracker is added)
@@ -864,12 +815,6 @@ class TrackerBox (gtk.VBox):
             selected_tracker_id = mod.get_value(itr, 0)
             self.tracker_delete_tracks(selected_tracker_id)
 
-    @simple_debug
-    def keep_cb (self, action):
-        mod, itr = self.tracker_tree.get_selection().get_selected()
-        selected_tracker_id = mod.get_value(itr, 0)
-        self.tracker_keep_tracks(selected_tracker_id)
-
     @simple_debug
     def tracker_delete_tracks (self, tracker_id):
         clearer = Undo.UndoableObject(
@@ -878,15 +823,6 @@ class TrackerBox (gtk.VBox):
             self.main_ui.history)
         clearer.perform()
 
-    @simple_debug
-    def tracker_keep_tracks (self, tracker_id):
-        clearer = Undo.UndoableObject(
-            lambda *args: self.main_ui.cleared.append(self.main_ui.gsd.delete_except_for_tracker(tracker_id)),
-            lambda *args: [self.main_ui.gsd.add_value(*entry) for entry in self.main_ui.cleared.pop()],
-            self.main_ui.history)
-        clearer.perform()
-
-
 def start_game ():
     if options.debug:
         print 'Starting GNOME Sudoku in debug mode'
