@@ -3,6 +3,7 @@
 
 import gtk, gobject, pango, cairo
 import math
+import timer
 from gettext import gettext as _
 
 ERROR_HIGHLIGHT_COLOR = (1.0, 0, 0)
@@ -15,7 +16,6 @@ BASE_FONT_SIZE = pango.SCALE * 13
 NOTE_FONT_SIZE = pango.SCALE * 6
 
 BORDER_WIDTH = 9.0 # The size of space we leave for a box
-BORDER_LINE_WIDTH = 4 # The size of the line we draw around a selected box
 NORMAL_LINE_WIDTH = 1 # The size of the line we draw around a box
 
 class NumberSelector (gtk.EventBox):
@@ -98,6 +98,8 @@ class NumberBox (gtk.Widget):
     def __init__ (self, upper = 9, text = ''):
         gtk.Widget.__init__(self)
         self.upper = upper
+        self.parent_win = None
+        self.timer = None
         self.font = self.style.font_desc
         self.font.set_size(BASE_FONT_SIZE)
         self.note_font = self.font.copy()
@@ -113,6 +115,12 @@ class NumberBox (gtk.Widget):
         self.connect('motion-notify-event', self.motion_notify_cb)
         self.set_text(text)
 
+    def set_parent_win(self, new_parent):
+        self.parent_win = new_parent
+
+    def set_timer(self, new_timer):
+        self.timer = new_timer
+
     def pointer_enter_cb (self, *args):
         if not self.is_focus():
             self.set_state(gtk.STATE_PRELIGHT)
@@ -185,26 +193,65 @@ class NumberBox (gtk.Widget):
             # Make sure we don't trigger on unplugging the A/C charger etc
             return
         txt = txt.replace('KP_', '')
-        if self.get_text() == txt:
-            # If there's no change, do nothing
-            return
-        if txt in ['0', 'Delete', 'BackSpace']:
+
+        # Add the new value if need be
+        if txt in [str(n) for n in range(1, self.upper+1)]:
+            if e.state & gtk.gdk.CONTROL_MASK:
+                self.add_note_text(txt, top = True)
+            elif e.state & gtk.gdk.MOD1_MASK:
+                self.remove_note_text(txt, top = True)
+            elif self.get_text() != txt:
+                # If there's no change, do nothing
+
+                # First do a removal event -- this is something of a
+                # kludge, but it works nicely with old code that was based
+                # on entries, which also behave this way (they generate 2
+                # events for replacing a number with a new number - a
+                # removal event and an addition event)
+
+                if self.get_text():
+                    self.set_text_interactive('')
+                # Then add
+                self.set_text_interactive(txt)
+        elif txt in ['0', 'Delete', 'BackSpace']:
             self.set_text_interactive('')
         elif txt in ['n', 'N']:
-            self.show_note_editor(top = True)
+            if e.state & gtk.gdk.MOD1_MASK:
+                self.set_note_text_interactive(top_text = '')
+            else:
+                self.show_note_editor(top = True)
         elif txt in ['m', 'M']:
-            self.show_note_editor(top = False)
-        # And then add the new value if need be
-        elif txt in [str(n) for n in range(1, self.upper+1)]:
-            # First do a removal event -- this is something of a
-            # kludge, but it works nicely with old code that was based
-            # on entries, which also behave this way (they generate 2
-            # events for replacing a number with a new number - a
-            # removal event and an addition event)
-            if self.get_text():
-                self.set_text_interactive('')
-            # Then add
-            self.set_text_interactive(txt)
+            if e.state & gtk.gdk.MOD1_MASK:
+                self.set_note_text_interactive(bottom_text = '')
+            else:
+                self.show_note_editor(top = False)
+
+    def add_note_text(self, txt, top = False):
+        if top:
+            note = self.top_note_text
+        else:
+            note = self.bottom_note_text
+        if txt not in note:
+            tmp = list(note)
+            tmp.append(txt)
+            tmp.sort()
+            note = ''.join(tmp)
+            if top:
+                self.set_note_text_interactive(top_text = note)
+            else:
+                self.set_note_text_interactive(bottom_text = note)
+
+    def remove_note_text(self, txt, top = False):
+        if top:
+            note = self.top_note_text
+        else:
+            note = self.bottom_note_text
+        if txt in note:
+            note = note.replace(txt,'')
+            if top:
+                self.set_note_text_interactive(top_text = note)
+            else:
+                self.set_note_text_interactive(bottom_text = note)
 
     def note_changed_cb (self, w, top = False):
         if top:
@@ -212,6 +259,14 @@ class NumberBox (gtk.Widget):
         else:
             self.set_note_text_interactive(bottom_text = w.get_text())
 
+    def note_focus_in(self, win, evt):
+        if (self.timer):
+            self.timer.resume_timing()
+
+    def note_focus_out(self, wgt, evt):
+        if (self.timer):
+            self.timer.pause_timing()
+
     def show_note_editor (self, top = True):
         alloc = self.get_allocation()
         w = gtk.Window()
@@ -220,6 +275,8 @@ class NumberBox (gtk.Widget):
         w.set_decorated(False)
         w.set_position(gtk.WIN_POS_MOUSE)
         w.set_size_request(alloc.width, alloc.height/2)
+        if self.parent_win:
+            w.set_transient_for(self.parent_win)
         f = gtk.Frame()
         e = gtk.Entry()
         f.add(e)
@@ -229,7 +286,9 @@ class NumberBox (gtk.Widget):
             e.set_text(self.bottom_note_text)
         w.add(f)
         e.connect('changed', self.note_changed_cb, top)
+        e.connect('focus-in-event', self.note_focus_in)
         e.connect('focus-out-event', lambda e, ev, w: w.destroy(), w)
+        e.connect('focus-out-event', self.note_focus_out)
         e.connect('activate', lambda e, w: w.destroy(), w)
         x, y = self.window.get_origin()
         if top:
@@ -386,21 +445,20 @@ class NumberBox (gtk.Widget):
         # Create any resources in here.
         x, y, w, h = self.allocation
         cr = self.window.cairo_create()
+        self.draw_background_color(cr, w, h)
+        if self.is_focus():
+            self.draw_highlight_box(cr, w, h)
         if h < w:
             scale = h/float(BASE_SIZE)
         else:
             scale = w/float(BASE_SIZE)
         cr.scale(scale, scale)
-        self.draw_background_color(cr)
-        if self.is_focus():
-            self.draw_highlight_box(cr)
-        self.draw_normal_box(cr)
         self.draw_text(cr)
         if self.draw_boxes and self.is_focus():
             self.draw_note_area_highlight_box(cr)
 
 
-    def draw_background_color (self, cr):
+    def draw_background_color (self, cr, w, h):
         if self.read_only:
             if self.custom_background_color:
                 r, g, b = self.custom_background_color
@@ -418,55 +476,24 @@ class NumberBox (gtk.Widget):
                 self.style.base[self.state]
                 )
         cr.rectangle(
-            0, 0, BASE_SIZE, BASE_SIZE
+            0, 0, w, h,
             )
         cr.fill()
 
-    def draw_normal_box (self, cr):
-        state = self.state
-        if state == gtk.STATE_SELECTED:
-            # When the widget is selected, we still want the outer box to look normal
-            state = gtk.STATE_NORMAL
-        cr.set_source_color(
-            self.style.mid[state]
-            )
-        cr.rectangle(
-            NORMAL_LINE_WIDTH*0.5,
-            NORMAL_LINE_WIDTH*0.5,
-            BASE_SIZE-NORMAL_LINE_WIDTH,
-            BASE_SIZE-NORMAL_LINE_WIDTH,
-            )
-        cr.set_line_width(NORMAL_LINE_WIDTH)
-        cr.set_line_join(cairo.LINE_JOIN_ROUND)
-        cr.stroke()
-        # And now draw a thinner line around the very outside...
-        cr.set_source_color(
-            self.style.dark[state]
-            )
-        cr.rectangle(
-            NORMAL_LINE_WIDTH*0.25,
-            NORMAL_LINE_WIDTH*0.25,
-            BASE_SIZE-NORMAL_LINE_WIDTH*0.5,
-            BASE_SIZE-NORMAL_LINE_WIDTH*0.5,
-            )
-        cr.set_line_width(NORMAL_LINE_WIDTH*0.5)
-        cr.set_line_join(cairo.LINE_JOIN_MITER)
-        cr.stroke()
-
-    def draw_highlight_box (self, cr):
+    def draw_highlight_box (self, cr, w, h):
         cr.set_source_color(
             self.style.base[gtk.STATE_SELECTED]
             )
+        border = 4 * w / BASE_SIZE
         cr.rectangle(
             # left-top
-            BORDER_LINE_WIDTH*0.5,
-            BORDER_LINE_WIDTH*0.5,
+            border*0.5,
+            border*0.5,
             # bottom-right
-            BASE_SIZE-(BORDER_LINE_WIDTH),
-            BASE_SIZE-(BORDER_LINE_WIDTH),
+            w-border,
+            h-border,
             )
-        cr.set_line_width(BORDER_LINE_WIDTH)
-        cr.set_line_join(cairo.LINE_JOIN_ROUND)
+        cr.set_line_width(border)
         cr.stroke()
 
     def draw_note_area_highlight_box (self, cr):
@@ -475,7 +502,6 @@ class NumberBox (gtk.Widget):
             self.style.mid[self.state]
             )
         cr.set_line_width(NORMAL_LINE_WIDTH)
-        cr.set_line_join(cairo.LINE_JOIN_ROUND)
         # top rectangle
         cr.rectangle(NORMAL_LINE_WIDTH*0.5,
                      NORMAL_LINE_WIDTH*0.5,
@@ -565,6 +591,7 @@ class NumberBox (gtk.Widget):
 class SudokuNumberBox (NumberBox):
 
     normal_color = None
+    error_color = (1.0, 0, 0)
     highlight_color = ERROR_HIGHLIGHT_COLOR
 
     def set_color (self, color):
@@ -576,7 +603,7 @@ class SudokuNumberBox (NumberBox):
 
     def set_error_highlight (self, val):
         if val:
-            self.set_text_color((1.0, 0, 0))
+            self.set_text_color(self.error_color)
         else:
             self.set_text_color(self.normal_color)
 
@@ -594,8 +621,13 @@ class SudokuNumberBox (NumberBox):
 
     def set_impossible (self, val):
         if val:
-            self.set_text('X')
-        else: self.set_text('')
+            if not self.get_text():
+                self.set_text('X')
+                self.set_text_color(self.error_color)
+        elif self.get_text() == 'X':
+            self.set_text('')
+            self.set_text_color(self.normal_color)
+        self.queue_draw()
 
 
 gobject.type_register(NumberBox)
