@@ -36,6 +36,7 @@
 #include <libgames-support/games-scores.h>
 #include <libgames-support/games-scores-dialog.h>
 #include <libgames-support/games-stock.h>
+#include <libgames-support/games-fullscreen-action.h>
 
 #ifdef WITH_SMCLIENT
 #include <libgames-support/eggsmclient.h>
@@ -73,7 +74,6 @@ GamesPreimage *tiles_preimage;
 
 GtkActionGroup *action_group;
 GtkAction *fullscreen_action;
-GtkAction *leave_fullscreen_action;
 
 gboolean clear_buffer = TRUE;
 gboolean clear_game = TRUE;
@@ -156,9 +156,6 @@ void game_score (void);
 gint game_over (void);
 
 gboolean window_resize_cb (GtkWidget *, GdkEventConfigure *, gpointer);
-static void set_fullscreen_actions (gboolean is_fullscreen);
-static void fullscreen_cb (GtkAction * action);
-static gboolean window_state_cb (GtkWidget * widget, GdkEventWindowState * event);
 
 void new_game (gint requested_level);
 
@@ -429,10 +426,6 @@ const GtkActionEntry entries[] = {
   {"Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_game_cb)},
   {"Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL, G_CALLBACK (help_cb)},
   {"About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (about_cb)},
-  {"Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL,
-   G_CALLBACK (fullscreen_cb)},
-  {"LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL,
-   G_CALLBACK (fullscreen_cb)},
   {"Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL, G_CALLBACK (score_cb)}
 };
 
@@ -454,7 +447,6 @@ const char ui_description[] =
   "    </menu>"
   "    <menu action='ViewMenu'>"
   "      <menuitem action='Fullscreen'/>"
-  "      <menuitem action='LeaveFullscreen'/>"
   "    </menu>"
   "    <menu action='HelpMenu'>"
   "      <menuitem action='Contents'/>"
@@ -488,7 +480,9 @@ main (int argc, char **argv)
   if (!games_runtime_init ("gnotski"))
     return 1;
 
+#ifdef ENABLE_SETGID
   setgid_io_init ();
+#endif
 
   context = g_option_context_new (NULL);
 #if GLIB_CHECK_VERSION (2, 12, 0)
@@ -540,8 +534,6 @@ main (int argc, char **argv)
 
   g_signal_connect (window, "delete_event",
 		    G_CALLBACK (quit_game_cb), NULL);
-  g_signal_connect (window, "window_state_event",
-		    G_CALLBACK (window_state_cb), NULL);
 
 
   vbox = gtk_vbox_new (FALSE, 0);
@@ -580,8 +572,8 @@ expose_space (GtkWidget * widget, GdkEventExpose * event)
   if (clear_game)
     return FALSE;
 
-  gdk_draw_drawable (widget->window,
-		     widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+  gdk_draw_drawable (gtk_widget_get_window (widget),
+		     gtk_widget_get_style (widget)->fg_gc[GTK_STATE_NORMAL],
 		     buffer, event->area.x, event->area.y,
 		     event->area.x, event->area.y,
 		     event->area.width, event->area.height);
@@ -603,37 +595,6 @@ redraw_all (void)
   return FALSE;
 }
 
-
-static void
-set_fullscreen_actions (gboolean is_fullscreen)
-{
-  gtk_action_set_sensitive (leave_fullscreen_action, is_fullscreen);
-  gtk_action_set_visible (leave_fullscreen_action, is_fullscreen);
-
-  gtk_action_set_sensitive (fullscreen_action, !is_fullscreen);
-  gtk_action_set_visible (fullscreen_action, !is_fullscreen);
-}
-
-static void
-fullscreen_cb (GtkAction * action)
-{
-  if (action == fullscreen_action) {
-    gtk_window_fullscreen (GTK_WINDOW (window));
-  } else {
-    gtk_window_unfullscreen (GTK_WINDOW (window));
-  }
-}
-
-/* Just in case something else takes us to/from fullscreen. */
-static gboolean
-window_state_cb (GtkWidget * widget, GdkEventWindowState * event)
-{
-  if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
-    set_fullscreen_actions (event->new_window_state &
-			    GDK_WINDOW_STATE_FULLSCREEN);
-  return FALSE;
-}
-
 static gboolean
 movable (gint id)
 {
@@ -712,9 +673,9 @@ gui_draw_space (void)
   GtkStyle *style;
 
   if (!backgc)
-    backgc = gdk_gc_new (space->window);
+    backgc = gdk_gc_new (gtk_widget_get_window (space));
   if (!bordergc)
-    bordergc = gdk_gc_new (space->window);
+    bordergc = gdk_gc_new (gtk_widget_get_window (space));
 
   style = gtk_widget_get_style (space);
 
@@ -731,7 +692,7 @@ gui_draw_space (void)
   if (buffer)
     g_object_unref (buffer);
 
-  buffer = gdk_pixmap_new (space->window,
+  buffer = gdk_pixmap_new (gtk_widget_get_window (space),
 			   width * tile_size + SPACE_PADDING,
 			   height * tile_size + SPACE_PADDING, -1);
 
@@ -1071,12 +1032,8 @@ create_menubar (void)
   gtk_action_group_add_actions (action_group, entries,
 				G_N_ELEMENTS (entries), window);
 
-  fullscreen_action =
-    gtk_action_group_get_action (action_group, "Fullscreen");
-  leave_fullscreen_action =
-    gtk_action_group_get_action (action_group, "LeaveFullscreen");
-
-  set_fullscreen_actions (FALSE);
+  fullscreen_action = GTK_ACTION (games_fullscreen_action_new ("Fullscreen", GTK_WINDOW (window)));
+  gtk_action_group_add_action_with_accel (action_group, fullscreen_action, NULL);
 
   ui_manager = gtk_ui_manager_new ();
   gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
@@ -1455,7 +1412,7 @@ save_state_cb (EggSMClient *client,
   gint argc;
   gint xpos, ypos;
 
-  gdk_window_get_origin (window->window, &xpos, &ypos);
+  gdk_window_get_origin (gtk_widget_get_window (window), &xpos, &ypos);
 
   argc = 0;
   argv[argc++] = g_get_prgname ();
